/* $Id: cursesmessagedisplay.C,v 1.3 2003/09/14 19:20:38 mrsam Exp $
**
** Copyright 2003, Double Precision Inc.
**
** See COPYING for distribution information.
*/

#include "config.h"
#include "mymessage.H"
#include "myfolder.H"
#include "myserver.H"
#include "myservercallback.H"
#include "myserverpromptinfo.H"
#include "cursesmessage.H"
#include "cursesmessagedisplay.H"
#include "gettext.H"
#include "gpg.H"
#include "passwordlist.H"
#include "globalkeys.H"
#include "addressbook.H"
#include "disconnectcallbackstub.H"
#include "curses/cursestitlebar.H"
#include "curses/cursescontainer.H"
#include "curses/cursesstatusbar.H"

#include "libmail/smtpinfo.H"

#include <sstream>

using namespace std;

extern Gettext::Key key_LESSTHAN;
extern Gettext::Key key_REPLACE;

extern Gettext::Key key_TAKEADDR;
extern Gettext::Key key_FOLDERINDEX;
extern Gettext::Key key_DELETE;
extern Gettext::Key key_UNDELETE;
extern Gettext::Key key_NEXTMESSAGE;
extern Gettext::Key key_PREVMESSAGE;
extern Gettext::Key key_VIEWATT;
extern Gettext::Key key_BOUNCE;
extern Gettext::Key key_REPLY;
extern Gettext::Key key_FWD;
extern Gettext::Key key_HEADERS;
extern Gettext::Key key_UNENCRYPT;

extern CursesTitleBar *titleBar;
extern CursesStatusBar *statusBar;

extern void folderIndexScreen(void *);
extern void hierarchyScreen(void *);
extern void showAttachments(void *);

static void goNextMessage(void *vp)
{
	myFolder *f=(myFolder *)vp;
	size_t messageNum;

	f->getNextMessage(messageNum);
	f->goMessage(messageNum, "");
}

static void goNextUnreadMessage(void *vp)
{
	myFolder *f=(myFolder *)vp;
	size_t messageNum;

	f->getNextUnreadMessage(messageNum);
	f->goMessage(messageNum, "");
}

static void goPrevMessage(void *vp)
{
	myFolder *f=(myFolder *)vp;
	size_t messageNum;

	f->getPrevMessage(messageNum);
	f->goMessage(messageNum, "");
}

CursesMessageDisplay::CursesMessageDisplay(CursesContainer *parent,
				    CursesMessage *messageArg)
	: Curses(parent), CursesKeyHandler(PRI_SCREENHANDLER),
	  messageInfoPtr(messageArg)
{
	CursesMessage::currentDisplay=this;

	resized();
	folderUpdated();
}

void CursesMessageDisplay::folderUpdated()
{
	string inProgress="----";

	CursesMessage *messageInfo=messageInfoPtr;

	if (!messageInfo)
		return;

	if (!messageInfo->reformatting()) // Reformat finished
	{
		size_t nLines=messageInfo->nLines();
		size_t h=getHeight();

		size_t lastLine= nLines > h ? nLines-h:0;

		string buffer;

		{
			ostringstream o;

			o << (lastLine == 0 ? 100
			      : firstLineShown * 100 / lastLine);
			buffer=o.str();
		}
		buffer += "%";

		inProgress=buffer;
	}

	titleBar->setTitles(Gettext(_("%1%: %2% of %3% %4%"))
			    << messageInfo->myfolder->getFolder()->getName()
			    << messageInfo->messagesortednum+1
			    << messageInfo->myfolder->getServer()
			    ->server->getFolderIndexSize()
			    << (messageInfo->orig_status_code == 'N'
				? _(" (NEW)"):
				messageInfo->orig_status_code == 'D'
				? _(" (DELETED)"): ""),
			    inProgress);
}

CursesMessageDisplay::~CursesMessageDisplay()
{
	CursesMessage::currentDisplay=NULL;
}

int CursesMessageDisplay::getWidth() const
{
	return getScreenWidth();
}

int CursesMessageDisplay::getHeight() const
{
	return (getParent()->getHeight());
}

void CursesMessageDisplay::draw()
{
	int h=getHeight();

	int i;

	for (i=0; i<h; i++)
		drawLine(firstLineShown+i);
}

void CursesMessageDisplay::drawLine(size_t lineNum)
{
	if (lineNum < firstLineShown)
		return;

	if (lineNum >= firstLineShown + getHeight())
		return;

	size_t w=getWidth();

	CursesMessage *messageInfo=messageInfoPtr;

	if (!messageInfo)
	{
		Curses::CursesAttr attr;
		vector<wchar_t> l;

		l.insert(l.end(), w, ' ');
		l.push_back(0);
		writeText(&l[0], lineNum - firstLineShown, 0, attr);
		return;
	}

	messageInfo->drawLine(lineNum, this, lineNum-firstLineShown);
}

void CursesMessageDisplay::resized()
{
	CursesMessage *messageInfo=messageInfoPtr;

	firstLineShown=0;

	if (messageInfo)
		messageInfo->beginReformat(getWidth());
	draw();
}

bool CursesMessageDisplay::isFocusable()
{
	return true;
}

bool CursesMessageDisplay::processKeyInFocus(const Curses::Key &key)
{
	CursesMessage *messageInfo=messageInfoPtr;

	if (!messageInfo)
		return false;

	if (key == key_TAKEADDR)
	{
		vector<mail::address> addrList;

		addrList.reserve(messageInfo->envelope.from.size() +
				 messageInfo->envelope.to.size() +
				 messageInfo->envelope.cc.size() +
				 messageInfo->envelope.bcc.size() +
				 messageInfo->envelope.sender.size() +
				 messageInfo->envelope.replyto.size());

		addrList.insert(addrList.end(),
				messageInfo->envelope.from.begin(),
				messageInfo->envelope.from.end());

		addrList.insert(addrList.end(),
				messageInfo->envelope.to.begin(),
				messageInfo->envelope.to.end());

		addrList.insert(addrList.end(),
				messageInfo->envelope.cc.begin(),
				messageInfo->envelope.cc.end());

		addrList.insert(addrList.end(),
				messageInfo->envelope.bcc.begin(),
				messageInfo->envelope.bcc.end());

		addrList.insert(addrList.end(),
				messageInfo->envelope.sender.begin(),
				messageInfo->envelope.sender.end());

		addrList.insert(addrList.end(),
				messageInfo->envelope.replyto.begin(),
				messageInfo->envelope.replyto.end());

		AddressBook::take(addrList);
		return true;
	}
	if (key == key_FOLDERINDEX ||
	    key == key_LESSTHAN)
	{
		keepgoing=false;
		myServer::nextScreen=&folderIndexScreen;
		myServer::nextScreenArg=messageInfo->myfolder;
		return true;
	}


	if (key == key_DELETE)
		messageInfo->myfolder->markDeleted( messageInfo->messagesortednum,
						    true, false);

	if (key == key_UNDELETE)
		messageInfo->myfolder->markDeleted( messageInfo->messagesortednum,
						    false, false);

	if (key == key_NEXTMESSAGE ||
	    key == key_DELETE ||
	    key == key_UNDELETE)
	{
		size_t dummy;

		if (messageInfo->myfolder->getNextMessage(dummy))
		{
			Curses::keepgoing=false;
			myServer::nextScreen= &goNextMessage;
			myServer::nextScreenArg=messageInfo->myfolder;
			return true;
		}
		return true;
	}

	if (key == key_PREVMESSAGE)
	{
		size_t dummy;

		if (messageInfo->myfolder->getPrevMessage(dummy))
		{
			Curses::keepgoing=false;
			myServer::nextScreen= &goPrevMessage;
			myServer::nextScreenArg=messageInfo->myfolder;
		}
		return true;
	}

	if (key == ' ' || key == Key::PGDN || key == Key::DOWN)
	{
		size_t nLines=messageInfo->nLines();
		size_t h=getHeight();

		size_t lastLine= nLines > h ? nLines-h:0;

		size_t firstLineSaved=firstLineShown;

		if (key == Key::DOWN)
			++firstLineShown;
		else
			firstLineShown += h;

		if (firstLineShown > lastLine)
			firstLineShown=lastLine;

		if (firstLineSaved == firstLineShown && key == ' ')
		{
			size_t dummy;

			if (messageInfo->myfolder->
			    getNextUnreadMessage(dummy))
			{
				Curses::keepgoing=false;
				myServer::nextScreen= &goNextUnreadMessage;
				myServer::nextScreenArg=messageInfo->myfolder;
				return true;
			}

			statusBar->clearstatus();
			statusBar->status(Gettext(_("No more unread mail in %1%"))
					  << messageInfo->myfolder
					  ->getFolder()->getName());
			return true;
		}

		folderUpdated();
		draw();
		return true;
	}

	if (key == Key::PGUP || key == Key::UP)
	{
		size_t h=key == Key::UP ? 1:getHeight();

		firstLineShown -= firstLineShown < h ? firstLineShown:h;
		folderUpdated();
		draw();
		return true;
	}

	if (key == key_VIEWATT)
	{
		Curses::keepgoing=false;
		myServer::nextScreen=showAttachments;
		myServer::nextScreenArg=messageInfo;
		return true;
	}

	if (key == key_BOUNCE)
	{
		if (messageInfo->shownMimeId.size())
		{
			statusBar->clearstatus();
			statusBar->status(_("Cannot forward only an attachment, just the whole message."));
			statusBar->beepError();
			return true;
		}

		mail::smtpInfo sendInfo;

		string from;
		string replyto;
		string fcc;
		string customheaders;

		if (!myMessage::getDefaultHeaders(messageInfo->myfolder
						  ->getFolder(),
						  messageInfo->myfolder
						  ->getServer(),
						  from, replyto, fcc,
						  customheaders))
		{
			return true;
		}

		{
			vector<mail::address> addrList;
			size_t errIndex;

			if (mail::address::fromString(from, addrList,
						      errIndex)
			    && addrList.size() > 0)
				sendInfo.sender=addrList[0].addr;
		}

		mail::ptr<myFolder> folderPtr=messageInfo->myfolder;

		if (!CursesMessage::getBounceTo(sendInfo)
		    || !CursesMessage::getSendInfo(Gettext(_("Blind-forward message to %1% address(es)? (Y/N) "))
						   << sendInfo.recipients
						   .size(), "",
						   sendInfo, NULL))
			return true;

		disconnectCallbackStub disconnectStub;
		myServer::Callback sendCallback;

		mail::account *smtpServer;
		mail::folder *folder=
			CursesMessage::getSendFolder(sendInfo, smtpServer,
						     NULL,
						     disconnectStub);

		if (!folder)
			return true;

		if (folderPtr.isDestroyed() || messageInfoPtr.isDestroyed())
		{
			delete folder;
			if (smtpServer)
				delete smtpServer;
			return true;
		}

		try {
			messageInfo->copyContentsTo(folder, sendCallback);

			bool rc=myServer::eventloop(sendCallback);

			delete folder;
			folder=NULL;

			if (smtpServer)
			{
				if (rc)
				{
					myServer::Callback disconnectCallback;

					disconnectCallback.noreport=true;

					smtpServer->logout(disconnectCallback);
					myServer::
						eventloop(disconnectCallback);
				}

				delete smtpServer;
				smtpServer=NULL;
			}
			

		} catch (...) {

			if (folder)
				delete folder;
			if (smtpServer)
				delete smtpServer;
			LIBMAIL_THROW();
		}

		return true;
	}

	if (key == key_REPLY)
	{
		mail::ptr<myFolder> folder=messageInfo->myfolder;

		if (myMessage::checkInterrupted() &&
		    !folder.isDestroyed() &&
		    !messageInfoPtr.isDestroyed())
			messageInfo->reply();
		return true;
	}

	if (key == key_FWD)
	{
		mail::ptr<myFolder> folder=messageInfo->myfolder;

		if (myMessage::checkInterrupted() &&
		    !folder.isDestroyed() &&
		    !messageInfoPtr.isDestroyed())
			messageInfo->forward();
		return true;
	}

	if (key == key_HEADERS)
	{
		erase();
		messageInfo->fullEnvelopeHeaders=
			!messageInfo->fullEnvelopeHeaders;
		firstLineShown=0;
		messageInfo->beginReformat(getWidth());
		folderUpdated();
		return true;
	}

	if (key == key_REPLACE)
	{
		erase();
		messageInfo->rot13=!messageInfo->rot13;
		firstLineShown=0;
		messageInfo->beginReformat(getWidth());
		folderUpdated();
		return true;
	}

	if ((key == key_UNENCRYPT) &&
	    (messageInfo->isSigned() ||
	     messageInfo->isEncrypted()) &&
	    GPG::gpg.gpg_installed())
	{
		mail::ptr<CursesMessage> ptr=messageInfo;
		string passphrase;
		vector<string> options;
		bool wasEncrypted=messageInfo->isEncrypted();

		if (wasEncrypted)
		{
			if (!PasswordList::passwordList.check("decrypt:",
							      passphrase))
			{

				myServer::promptInfo
					passPrompt(_("Passphrase (if"
						     " required): "));

				passPrompt=myServer::prompt(passPrompt
							    .password());

				if (passPrompt.abortflag || ptr.isDestroyed())
					return true;

				passphrase=passPrompt;
			}
		}

		string::iterator b=GPG::gpg.extraDecryptVerifyOptions.begin();

		while (b != GPG::gpg.extraDecryptVerifyOptions.end())
		{
			if (isspace((int)(unsigned char)*b))
			{
				b++;
				continue;
			}

			string::iterator s=b;

			while (b != GPG::gpg.extraDecryptVerifyOptions.end())
			{
				if (isspace((int)(unsigned char)*b))
					break;
				b++;
			}

			options.push_back(string(s, b));
		}

		bool decryptFailed;

		if (messageInfo->decrypt(passphrase, options, decryptFailed))
		{
			if (wasEncrypted && !decryptFailed)
				PasswordList::passwordList
					.save("decrypt:", passphrase);

			erase();
			firstLineShown=0;
			messageInfo->beginReformat(getWidth());
			folderUpdated();

			if (wasEncrypted && decryptFailed)
			{
				statusBar->clearstatus();
				statusBar->status(_("ERROR: Decryption failed,"
						    " passphrase forgotten."));
				statusBar->beepError();
			}
		}

		return true;
	}

	return false;
}

int CursesMessageDisplay::getCursorPosition(int &row, int &col)
{
	Curses::getCursorPosition(row, col);
	return 0;
}

bool CursesMessageDisplay::processKey(const Curses::Key &key)
{
	CursesMessage *messageInfo=messageInfoPtr;

	if (!messageInfo)
		return false;

	return GlobalKeys::processKey(key, GlobalKeys::MESSAGESCREEN,
				      messageInfo->myfolder->getServer());
}

bool CursesMessageDisplay::listKeys( vector< pair<string, string> > &list)
{
	GlobalKeys::listKeys(list, GlobalKeys::MESSAGESCREEN);

	list.push_back(make_pair(Gettext::keyname(_("TAKEADDR_K:T")),
				 _("Take Addr")));

	list.push_back(make_pair(Gettext::keyname(_("FOLDERINDEX_K:I")),
				 _("Index")));

	list.push_back(make_pair(Gettext::keyname(_("BOUNCE_K:B")),
				 _("Blind Fwd")));

	list.push_back(make_pair(Gettext::keyname(_("FWD_K:F")),
				 _("Fwd")));

	list.push_back(make_pair(Gettext::keyname(_("HEADERS_K:H")),
				 _("Full Hdrs")));

	list.push_back(make_pair(Gettext::keyname(_("NEXT_K:N")),
				 _("Next")));

	list.push_back(make_pair(Gettext::keyname(_("PREV_K:P")),
				 _("Prev")));

	list.push_back( make_pair(Gettext::keyname(_("DELETE_K:D")),
				 _("Delete")));

	list.push_back( make_pair(Gettext::keyname(_("UNDELETE_K:U")),
				 _("Undelete")));

	list.push_back(make_pair(Gettext::keyname(_("REPLY_K:R")),
				 _("Reply")));

	list.push_back(make_pair(Gettext::keyname(_("ROT13_K:^R")),
				 _("Rot 13")));

	list.push_back(make_pair(Gettext::keyname(_("VIEWATT_K:V")),
				 _("View Attchs")));

	list.push_back(make_pair(Gettext::keyname(_("SPACE_K:SP")),
				 _("Next Unread")));

	if (!messageInfoPtr.isDestroyed() &&
	    (messageInfoPtr->isSigned() ||
	     messageInfoPtr->isEncrypted()) &&
	    GPG::gpg.gpg_installed())
	{
		list.push_back(make_pair(Gettext::keyname(_("UNENCRYPT_K:Y")),
					 _("decrYpt/Vfy")));
	}
	return false;
}


