/* $Id: addmessage.H,v 1.1 2003/05/27 14:09:08 mrsam Exp $
**
** Copyright 2002, Double Precision Inc.
**
** See COPYING for distribution information.
*/
#ifndef libmail_addmessage_H
#define libmail_addmessage_H

#include "mail.H"

#include <string>

#include <time.h>

#include "objectmonitor.H"
#include "namespace.H"

LIBMAIL_START

////////////////////////////////////////////////////////////////////////////
//
// Add message to a folder.  Each mail account is expected to create a
// subclass of mail::addMessage.  A mail::addMessage object gets created by
// mail::folder::addMessage().  This superclass provides fields, initialized
// with defaults.  The fields may be changed at any time before invoking the
// go() method.  The message text is provided by calling saveMessageContents().
// saveMessageContents() may be called repeatedly to provide the contents of
// a large message in pieces.  The message is saved by the go() method.
// The process may be aborted at any time, prior to go(), by invoking fail(),
// which automatically destroys the mail::addMessage object.
//
// A failure occuring while trying to add the message (including invoking
// fail()) is reported by the callback's fail method (the subclass receives
// the callback object and is responsible for dispatching the proper
// notification).


class addMessage : private ptr<mail::account> {

protected:
	bool checkServer();

public:
	addMessage(mail::account *);
	virtual ~addMessage();

	mail::messageInfo messageInfo; // Message flags

	time_t messageDate;	// Message add date.

	virtual void saveMessageContents(std::string)=0;
	virtual void go()=0;
	virtual void fail(std::string errmsg)=0;
};

// addMessage is a "push" interface - the application "pushes"
// the new message's contents via saveMessageContents().  The mail::ACCOUNT
// API provides an alternative "pull" implementation, where a
// addMessagePull object is passed instead, whose getMessageContents()
// method is repeatedly invoked.  getMessageContents() should return the
// next part of the message's contents.  getMessageContents() is called
// repeatedly, until it returns an empty string which signifies end of
// message contents.

class addMessagePull {
public:
	addMessagePull();
	virtual ~addMessagePull();

	mail::messageInfo messageInfo; // Message flags
	time_t messageDate;	// Message add date.

	virtual std::string getMessageContents()=0;
};

LIBMAIL_END

#endif

