/* $Id: addressbookadd.C,v 1.1 2003/05/27 14:09:08 mrsam Exp $
**
** Copyright 2002, Double Precision Inc.
**
** See COPYING for distribution information.
*/
#include "libmail_config.h"
#include "addressbookadd.H"
#include "rfc2047encode.H"
#include "envelope.H"
#include "rfcaddr.H"
#include "unicode/unicode.h"
#include <ctype.h>
#include <vector>

using namespace std;

mail::addressbook::Add::Add(mail::addressbook *addressBookArg,
			    mail::addressbook::Entry entryArg,
			    string oldUidArg,
			    mail::callback &callbackArg)
	: addressBook(addressBookArg),
	  newEntry(entryArg),
	  oldUid(oldUidArg),
	  callback(callbackArg),
	  addMessage(NULL)
{
}

void mail::addressbook::Add::go()
{
	string nickname=toutf8(newEntry.nickname);
	string name=toutf8(newEntry.name);
	string address=toutf8(newEntry.address);

	string::iterator b=nickname.begin(), e=nickname.end();

	while (b != e)
	{
		char c= *b++;

		if ( (int)(unsigned char)c < ' ' || c == '[' || c == ']')
			fail("Invalid address book nickname.");
	}

	vector<mail::address> addrList;

	b=address.begin();
	e=address.end();

	while (b != e)
	{
		char c= *b;

		if (c == ',' || isspace((int)(unsigned char)c))
		{
			b++;
			continue;
		}

		string::iterator start= b;

		while (b != e)
		{
			char c= *b;

			if (c == ',' || isspace((int)(unsigned char)c))
				break;

			b++;
		}

		addrList.push_back(mail::address(name, string(start, b)));
	}

	if (addrList.size() == 0)
	{
		fail("Invalid address.");
		return;
	}

	if (addrList.size() > 1)
	{
		vector<mail::address>::iterator b=addrList.begin(),
			e=addrList.end();

		while (b != e)
		{
			b->name="";
			b++;
		}
	}

	successFunc= &mail::addressbook::Add::added;

	mail::addMessage *addp=
		addressBook->folder->addMessage(*this);

	if (!addp)
		return;

	time(&addp->messageDate);

	addMessage=addp;

	try {
		addMessage->
			saveMessageContents("From: \"Libmail Address Book\" <libmail@localhost>\n"
					    "Subject: Address Book: "
					    + (string)
					    mail::rfc2047::encode("["
								  + nickname
								  + "]",
								  unicode_UTF8
								  .chset) +
					    "\n"
					    "Mime-Version: 1.0\n"
					    "Content-Type: multipart/mixed; boundary=\"libmail\"\n"
					    "\n"
					    "This is a multipart MIME message\n"
					    "\n"
					    "--libmail\n"
					    "Content-Type: text/plain; charset=iso-8859-1\n"
					    "Content-Transfer-Encoding: 8bit\n"
					    "\n"
					    "This message is used to store Libmail's address book.  Please do not modify\n"
					    "this folder, and message!\n"
					    "\n"
					    "--libmail\n"
					    "Content-Type: text/x-libmail-addressbook; charset=utf-8\n"
					    "Content-Transfer-Encoding: 8bit\n"

					    "\n" +
					    mail::address::toString("Address: "
								    ,
								    addrList) +
					    "\n\n--libmail--\n");

		addMessage->go();

	} catch (...) {
		addMessage->fail("An exception has occured while trying to update the address book.");

	}
}

mail::addressbook::Add::~Add()
{
}

void mail::addressbook::Add::success(string successMsg)
{
	(this->*successFunc)(successMsg);
}

//
// After adding a new entry, make sure it gets added to the index.
//

void mail::addressbook::Add::added(string successMsg)
{
	successFunc= &mail::addressbook::Add::checked;

	addressBook->server->checkNewMail( *this );
}

//
// Now, update our address book index.

void mail::addressbook::Add::checked(string successMsg)
{
	size_t n=addressBook->index.size();

	size_t n2=addressBook->server->getFolderIndexSize();

	vector<size_t> msgNums;

	while (n < n2)
		msgNums.push_back(n++);

	if (msgNums.size() == 0)
	{
		reindexed(successMsg); // Unlikely
		return;
	}

	addressBook->index.insert(addressBook->index.end(),
				  msgNums.size(), Index());

	successFunc= &mail::addressbook::Add::reindexed;

	addressBook->server->readMessageAttributes(msgNums,
						   addressBook->server
						   -> ENVELOPE,
						   *this);
}

void mail::addressbook::Add::messageEnvelopeCallback(size_t messageNumber,
						     const mail::envelope
						     &envelope)
{
	addressBook->setIndex(messageNumber, envelope.subject);
}

void mail::addressbook::Add::reportProgress(size_t bytesCompleted,
					    size_t bytesEstimatedTotal,

					    size_t messagesCompleted,
					    size_t messagesEstimatedTotal)
{
	callback.reportProgress(bytesCompleted, bytesEstimatedTotal,
				messagesCompleted, messagesEstimatedTotal);
}

void mail::addressbook::Add::reindexed(string successMsg)
{
	try {
		// If this is meant to replace another entry, delete it then.

		if (oldUid.size() > 0)
		{
			addressBook->del(oldUid, callback);
		}
		else
			callback.success(successMsg);
		delete this;
	} catch (...) {
		delete this;
		LIBMAIL_THROW();
	}
}

void mail::addressbook::Add::fail(string failMsg)
{
	try {
		callback.fail(failMsg);
		delete this;
	} catch (...) {
		delete this;
		LIBMAIL_THROW();
	}
}

