/* $Id: addressbookget.H,v 1.1 2003/05/27 14:09:08 mrsam Exp $
**
** Copyright 2002, Double Precision Inc.
**
** See COPYING for distribution information.
*/
#ifndef libmail_addressbookget_H
#define libmail_addressbookget_H

#include "addressbook.H"
#include "structure.H"
#include "rfcaddr.H"

#include <map>

//
// Extract an address book entry, as follows:
//
// 1. Read the message's MIME structure.
// 2. Verify the presence of text/x-libmail-addressbook content.
// 3. Read text/x-libmail-addressbook content.

class mail::addressbook::GetAddressList : public mail::callback::message {

	mail::ptr<mail::addressbook> addressBook;
	size_t msgNum;
	std::vector<mail::address> &addrListRet;
	mail::callback &callback;
	const struct unicode_info *unicodeInfo;

	mail::mimestruct mimeStructure;

	void success(std::string successMsg);
	void (mail::addressbook::GetAddressList::*successFunc)(std::string);

	void readstructure(std::string successMsg);
	void readContents(std::string successMsg);

	void fail(std::string failMsg);

	std::string fromutf8(std::string);

	void messageStructureCallback(size_t messageNumber,
				      const mail::mimestruct
				      &messageStructure);
	void messageTextCallback(size_t n, std::string text);

	void reportProgress(size_t bytesCompleted,
			    size_t bytesEstimatedTotal,

			    size_t messagesCompleted,
			    size_t messagesEstimatedTotal);

	std::string linebuffer;
	// Reading x-libmail-addressbook content; partial line read.

	void addressBookLine(std::string text);

	std::map<std::string, std::string> addressBookLineMap;
	std::string lastAddressBookLine;
	// Name of the current x-libmail-addressbook header line being read.

public:
	GetAddressList(mail::addressbook *addressBookArg,
		       size_t msgNumArg,
		       std::vector<mail::address> &addrListRetArg,
		       mail::callback &callbackArg);
	~GetAddressList();

	void go();
};

#endif
