/* $Id: pop3maildrop.C,v 1.3 2003/11/30 15:55:47 mrsam Exp $
**
** Copyright 2002-2003, Double Precision Inc.
**
** See COPYING for distribution information.
*/
#include "libmail_config.h"
#include "pop3maildrop.H"
#include "pop3.H"
#include "maildiradd.H"
#include "driver.H"

#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>

using namespace std;

/////////////////////////////////////////////////////////////////////////////

LIBMAIL_START

static bool open_pop3maildrop(mail::account *&accountRet,
			 mail::account::openInfo &oi,
			 mail::callback &callback,
			 mail::callback::disconnect &disconnectCallback)
{
	mail::loginInfo pop3LoginInfo;

	if (!mail::pop3maildrop::loginUrlDecode(oi.url, pop3LoginInfo))
		return false;

	string pop3method;

	if (pop3LoginInfo.method == "pop3maildrop")
		pop3method="pop3";
	else if (pop3LoginInfo.method == "pop3maildrops")
		pop3method="pop3s";
	else
		return false;

	accountRet=new mail::pop3maildrop(disconnectCallback,
					  oi.loginCallbackObj,
					  callback,
					  oi.extraString,
					  pop3method +
					  oi.url.substr(oi.url.find(':')),
					  oi.pwd);

	return true;
}

static bool pop3maildrop_remote(string url, bool &flag)
{
	mail::loginInfo pop3LoginInfo;

	if (!mail::pop3maildrop::loginUrlDecode(url, pop3LoginInfo))
		return false;

	if (pop3LoginInfo.method != "pop3maildrop" &&
	    pop3LoginInfo.method != "pop3maildrops")
		return false;

	flag=false;
	return true;
}

driver pop3maildrop_driver= { &open_pop3maildrop, &pop3maildrop_remote };

LIBMAIL_END

/////////////////////////////////////////////////////////////////////////////
//
// Subclass pop3 to forego UIDL, and just push all msgs to the pop3maildrop
// maildir.

class mail::pop3maildrop::pop3acct : public mail::pop3,
	    private mail::callback {

	mail::pop3maildrop *myMaildrop;

public:
	pop3acct(mail::pop3maildrop *myMaildropArg,
		 std::string url, std::string passwd,
		 mail::loginCallback *loginCallbackFunc,
		 callback &callback,
		 callback::disconnect &disconnectCallback);
	~pop3acct();

	vector<mail::addMessage *> downloadedMsgs;

	bool ispop3maildrop();
	mail::addMessage *newDownloadMsg();
	string commitDownloadedMsgs();

private:
	bool errflag;
	string errmsg;

	// Inherited from mail::callback

	void success(std::string message);
	void fail(std::string message);

	void reportProgress(size_t bytesCompleted,
			    size_t bytesEstimatedTotal,

			    size_t messagesCompleted,
			    size_t messagesEstimatedTotal);


};


mail::pop3maildrop::pop3acct::pop3acct(mail::pop3maildrop *myMaildropArg,
				       std::string url, std::string passwd,
				       mail::loginCallback *loginCallbackFunc,
				       callback &callback,
				       callback::disconnect
				       &disconnectCallback)
	: pop3(url, passwd, loginCallbackFunc, callback, disconnectCallback),
	  myMaildrop(myMaildropArg)
{
}

mail::pop3maildrop::pop3acct::~pop3acct()
{
	ispop3maildrop(); // Cleanup
}

bool mail::pop3maildrop::pop3acct::ispop3maildrop()
{
	vector<mail::addMessage *>::iterator b=downloadedMsgs.begin(),
		e=downloadedMsgs.end();

	while (b != e)
	{
		if (*b)
			(*b)->fail("Cancelled.");
		*b=NULL;
		++b;
	}

	downloadedMsgs.clear();
	errflag=false;
	return true;
}

mail::addMessage *mail::pop3maildrop::pop3acct::newDownloadMsg()
{
	downloadedMsgs.push_back(NULL);

	return (downloadedMsgs.end()[-1]=new
		mail::maildir::addmessage(myMaildrop,
					  myMaildrop->path, *this));
}

string mail::pop3maildrop::pop3acct::commitDownloadedMsgs()
{
	vector<mail::addMessage *>::iterator b=downloadedMsgs.begin(),
		e=downloadedMsgs.end();

	while (b != e)
	{
		if (*b)
			(*b)->go();
		*b=NULL;
		++b;
	}
	downloadedMsgs.clear();
	if (!errflag)
		errmsg="";
	myMaildrop->maildir::checkNewMail(NULL);
	return errmsg;
}

void mail::pop3maildrop::pop3acct::success(std::string message)
{
}

void mail::pop3maildrop::pop3acct::fail(std::string message)
{
	errflag=true;
	errmsg=message;
}

void mail::pop3maildrop
::pop3acct::reportProgress(size_t bytesCompleted,
			   size_t bytesEstimatedTotal,
			   size_t messagesCompleted,
			   size_t messagesEstimatedTotal)
{
}

/////////////////////////////////////////////////////////////////////////////

mail::pop3maildrop::pop3maildrop(mail::callback::disconnect
				 &disconnect_callback,
				 mail::loginCallback *loginCallbackFunc,
				 mail::callback &callback,
				 std::string pathArg,
				 std::string pop3url,
				 std::string pop3pass)
	: maildir((mail::callback::disconnect &)*this),
	  myPop3Acct(NULL)
{
	struct stat stat_buf;

	// Automatically create the maildir, if it doesn't exist.

	if (stat(pathArg.c_str(), &stat_buf) < 0 && errno == ENOENT)
	{
		if (!mail::maildir::maildirmake(pathArg, false))
		{
			callback.fail(strerror(errno));
			return;
		}
	}

	if (!init(callback, pathArg))
		return;
	maildir::ispop3maildrop=true;

	mail::loginInfo pop3LoginInfo;

	if (!mail::fd::loginUrlDecode(pop3url, pop3LoginInfo) ||
	    (pop3LoginInfo.method != "pop3" &&
	     pop3LoginInfo.method != "pop3s"))
	{
		callback.fail("Invalid POP3 maildrop URL");
		return;
	}

	myPop3Acct=new pop3acct(this,
				pop3url,
				pop3pass,
				loginCallbackFunc,
				callback,
				disconnect_callback);

	if (!myPop3Acct)
		callback.fail(strerror(errno));
}


bool mail::pop3maildrop::hasCapability(string capability)
{
	if (capability == LIBMAIL_SINGLEFOLDER)
		return true;

	return maildir::hasCapability(capability);
}

string mail::pop3maildrop::getCapability(string capability)
{
	if (capability == LIBMAIL_SERVERTYPE)
	{
		return "pop3maildrop";
	}

	if (myPop3Acct)
	{
		if (capability == LIBMAIL_SERVERDESCR)
		{
			return myPop3Acct->getCapability(capability);
		}
	}

	return maildir::getCapability(capability);		
}


// Callbacks for maildir's disconnect/servererror logic:

void mail::pop3maildrop::disconnected(const char *errmsg)
{
}

void mail::pop3maildrop::servererror(const char *errmsg)
{
}

mail::pop3maildrop::~pop3maildrop()
{
	if (myPop3Acct)
		delete myPop3Acct;
}

void mail::pop3maildrop::logout(callback &callback)
{
	if (myPop3Acct)
		myPop3Acct->logout(callback);
	else
		maildir::logout(callback);
}

void mail::pop3maildrop::checkNewMail(callback &callback)
{
	if (folderCallback && strcasecmp(folderPath.c_str(), "INBOX") == 0 &&
	    myPop3Acct)
	{
		myPop3Acct->checkNewMail(callback);
		return;
	}

	mail::maildir::checkNewMail(callback);
}
