/* $Id: rfcaddr.H,v 1.1 2003/05/27 14:09:09 mrsam Exp $
**
** Copyright 2002, Double Precision Inc.
**
** See COPYING for distribution information.
*/
#ifndef libmail_rfcaddr_h
#define libmail_rfcaddr_h

#include "libmail_config.h"
#include "namespace.H"

#include <stdlib.h>
#include <vector>
#include <string>

//
// An RFC 2822 address, a name and user@domain.
//

LIBMAIL_START

class mail::address {
public:
	std::string name;
	std::string addr;

	// If addr.size() == 0 this is the obsolete rfc 822
	// group list notation, and name is either ";" or "name:"

	address(std::string n, std::string a);

	~address();

	// Convert myself to "name" <addr> format.

	std::string toString() const;

	//
	// Create an RFC 2822 address header, from a vector of addresss.
	//

	static	std::string toString(std::string hdr, // Name:
				const std::vector<address> &h,
				size_t w=76);  // Max length of lines.

	//
	// Create a vector of addresss from a string
	//

	static bool fromString(std::string addresses,
			       std::vector<address> &h,
			       size_t &errIndex);

	// Addresses are appended to h
	// Returns false if memory allocation failed or addresses is
	// syntactically invalid.  errIndex points where in addresses the
	// syntax error is (or is string::npos if memory allocation failed)

	std::string getAddress() const;
	// Return addr with domain portion converted to lowercase

	bool operator==(const address &) const;
	// Compare the addr portions only, ignoring domain case

	bool operator!=(const address &a) const
	{
		return !operator==(a);
	}

};

LIBMAIL_END

#endif
