/* $Id: structure.H,v 1.2 2003/06/11 22:36:32 mrsam Exp $
**
** Copyright 2002, Double Precision Inc.
**
** See COPYING for distribution information.
*/
#ifndef libmail_structure_H
#define libmail_structure_H

#include "libmail_config.h"

#include <string>
#include <vector>
#include <map>

#include "namespace.H"

struct unicode_info;

LIBMAIL_START

class envelope;
class xenvelope;

///////////////////////////////////////////////////////////////////////////
//
// The structure of a message is described by a tree of mail::mimestruct
// objects.  Each mail::mimestruct objects represents a MIME section in the
// message, and may contain pointers to mail::mimestruct objects for content
// sections if the MIME section is a multipart section.

class mimestruct {

public:

	// The Content-Type: and Content-Disposition: headers have an
	// optional parameter=value; list, which is parsed into a hash.
	//
	// The get() method fetches the parameter.  It's a pretty smart get()
	// method.  If a character set is provided, the parameter value is
	// mapped to the specified character set.  Additionally, the get()
	// method automatically handles RFC 2231-encoded attributes.
	// Ain't that cool?

	class parameterList {
	public:
		std::map<std::string, std::string> map;

		parameterList();
		~parameterList();

		void set(std::string name, std::string value);

		bool exists(std::string name);
		std::string get(std::string name,
			   const struct unicode_info *info=NULL);
	};

	std::string mime_id;	// An opaque identifier of this MIME section.


	std::string type, subtype;	// text, plain

	bool messagerfc822() const
	{
		return type == "MESSAGE" && subtype == "RFC822";
	}

	bool multipartsigned() const
	{
		return type == "MULTIPART" && subtype == "SIGNED";
	}

	bool multipartencrypted() const
	{
		return type == "MULTIPART" && subtype == "ENCRYPTED";
	}

	parameterList type_parameters;

	// <charset, iso-8859-1>

	std::string content_id;
	std::string content_description;
	std::string content_transfer_encoding;

	size_t content_size;
	size_t content_lines;

	std::string content_md5;
	std::string content_language;

	std::string content_disposition;

	parameterList content_disposition_parameters;

	mimestruct();
	~mimestruct();

	// Copy constructors, and assignment operators, are fully functional.

	mimestruct(const mimestruct &cpy);
	mimestruct &operator=(const mimestruct &cpy);

private:
	mail::envelope *message_rfc822_envelope;

	std::vector<mimestruct *> children;
	mimestruct *parent;

	void destroy();
public:

	// MESSAGE/RFC822 content also has an associated mail::envelope
	// object.  getEnvelope() automatically creates this object,
	// which is automatically destroyed by the destructor

	class mail::envelope &getEnvelope();
	const class mail::envelope &getEnvelope() const
	{
		return *message_rfc822_envelope;
	}

	mimestruct *addChild();
	size_t getNumChildren() const { return children.size(); }

	const mimestruct *getChild(size_t n)
		const { return children[n]; }

	const mimestruct *getParent() const { return parent; }

	mimestruct *getChild(size_t n) { return children[n]; }
	mimestruct *getParent() { return parent; }
};
LIBMAIL_END

#endif
